#include <ax12.h>
#include <BioloidController.h>
#include "poses.h"

//initializing variables
BioloidController bioloid = BioloidController(1000000);

const int SERVOCOUNT = 6;
int id;
int pos;
boolean IDCheck;
boolean RunCheck;
int pin1 =16;
int pin4 =4;
int pin6=6;
 int sequence =0;

//Initializing in/outputs
void setup(){
  //pinMode(0,INPUT);
  pinMode(pin1,INPUT); //Signal from PLC
  pinMode(pin4,OUTPUT); //Signal to other robot to move
  pinMode(pin6,INPUT); //Signal from other robot
  //initialize variables 
 
  id = 1;
  pos = 0;
  IDCheck = 1;
  RunCheck = 0;
  //open serial port
  Serial.begin(9600);
  delay (500);   
  Serial.println("###########################");    
  Serial.println("Serial Communication Established.");    


  MoveHome();

  MenuOptions();

  RunCheck = 1;
}

void loop(){

  // if the input from PLC is high, it runs the program
 // if (pin1 == HIGH){
   // if (1){
  // prog();
  
  // Reads from Serial input and start functions depending on the input
  int inByte = Serial.read();

  switch (inByte) {

  case '1':    
    ScanServo();
    break;

  case '2':    
    MoveHome();
    break;     

  case '3':    
    RelaxServos();
    break;     

  case '4':
    prog();
    break;

  } 

}

void ScanServo(){
  //This function scans each motor and writes the position they are in. 
  //Used as a kind of position teaching.
  id = 1;  
  Serial.println("###########################");
  Serial.println("Starting Servo Scanning Test.");
  Serial.println("###########################");

  while (id <= SERVOCOUNT){
    pos =  ax12GetRegister(id, 36, 2);
    Serial.print("Servo ID: ");
    Serial.println(id);
    Serial.print("Servo Position: ");
    Serial.println(pos);

    if (pos <= 0){
      Serial.println("###########################");
      Serial.print("ERROR! Servo ID: ");
      Serial.print(id);
      Serial.println(" not found. Please check connection and verify correct ID is set.");
      Serial.println("###########################"); 
      IDCheck = 0;
    }

    id = (id++)%SERVOCOUNT;
    delay(1000);
  }
  if (IDCheck == 0){
    Serial.println("###########################");
    Serial.println("ERROR! Servo ID(s) are missing from Scan. Please check connection and verify correct ID is set.");
    Serial.println("###########################");  
  }
  else{
    Serial.println("All servo IDs present.");
  }
  if (RunCheck == 1){
    MenuOptions();
  }

}


void MoveHome(){
  //This function moves the motors to the Home position, ready for the next task
  delay(100);                    // recommended pause
  bioloid.loadPose(Home);   // load the pose from FLASH, into the nextPose buffer
  bioloid.readPose();            // read in current servo positions to the curPose buffer
  Serial.println("###########################");
  Serial.println("Moving servos to Home position");
  Serial.println("###########################");    
  delay(1000);
  bioloid.interpolateSetup(1000); // setup for interpolation from current->next over 1/2 a second
  while(bioloid.interpolating > 0){  // do this while we have not reached our new pose
    bioloid.interpolateStep();     // move servos, if necessary. 
    delay(3);
  }
  if (RunCheck == 1){
    MenuOptions();
  }
}


void MenuOptions(){
  // Prints the possibilities in the switch case.
  Serial.println("###########################"); 
  Serial.println("Please enter option 1-5 to run individual tests again.");     
  Serial.println("1) Scan");        
  Serial.println("2) home"); 
  Serial.println("3) relax");     
  Serial.println("4) program");            
  Serial.println("###########################"); 
}
void RelaxServos(){
  //Relaxes the motors so they can be moved around
  id = 1;
  Serial.println("###########################");
  Serial.println("Relaxing Servos.");
  Serial.println("###########################");    
  while(id <= SERVOCOUNT){
    Relax(id);
    id = (id++)%SERVOCOUNT;
    delay(50);
  }

  if (RunCheck == 1){
    MenuOptions();
  }
  while (id <= SERVOCOUNT){
    pos =  ax12GetRegister(id, 36, 2);
    Serial.print("Servo ID: ");
    Serial.println(id);
    Serial.print("Servo Position: ");
    Serial.println(pos);

    if (pos <= 0){
      Serial.println("###########################");
      Serial.print("ERROR! Servo ID: ");
      Serial.print(id);
      Serial.println(" not found. Please check connection and verify correct ID is set.");
      Serial.println("###########################"); 
      IDCheck = 0;
    }
  }
}

void prog(){
  // This is the main program which will move the arm to the correct positions.
  // made with Switch - Case that gets incremented for each case.
  while(1){
    Serial.print("inden case");
    Serial.print(", sequence");
    switch (sequence) {

   case 0:
      while(digitalRead(pin1) == LOW){
        delay(100);
      }
      
      delay(100);                    // recommended pause
      bioloid.loadPose(POS1);   // load the pose from FLASH, into the nextPose buffer
      bioloid.readPose();            // read in current servo positions to the curPose buffer
      Serial.println("###########################");
      Serial.println("Moving servos to Home position");
      Serial.println("###########################");    
      delay(1000);
      bioloid.interpolateSetup(1000); // setup for interpolation from current->next over 1/2 a second
      while(bioloid.interpolating > 0){  // do this while we have not reached our new pose
        bioloid.interpolateStep();     // move servos, if necessary. 
        delay(6);
      }
      sequence = sequence +10;
      break;
      //////////////////// move to above object

    case 10:

      delay(100);                    // recommended pause
      bioloid.loadPose(POS2);   // load the pose from FLASH, into the nextPose buffer
      bioloid.readPose();            // read in current servo positions to the curPose buffer
      Serial.println("###########################");
      Serial.println("Moving servos to Home position");
      Serial.println("###########################");    
      delay(1000);
      bioloid.interpolateSetup(1000); // setup for interpolation from current->next over 1/2 a second
      while(bioloid.interpolating > 0){  // do this while we have not reached our new pose
        bioloid.interpolateStep();     // move servos, if necessary. 
        delay(6);
      }
      sequence = sequence +10;
      break;
      ///////////////////////////// move down to object

    case 20:


      delay(100);                    // recommended pause
      bioloid.loadPose(POS3);   // load the pose from FLASH, into the nextPose buffer
      bioloid.readPose();            // read in current servo positions to the curPose buffer
      Serial.println("###########################");
      Serial.println("Moving servos to Home position");
      Serial.println("###########################");    
      delay(1000);
      bioloid.interpolateSetup(1000); // setup for interpolation from current->next over 1/2 a second
      while(bioloid.interpolating > 0){  // do this while we have not reached our new pose
        bioloid.interpolateStep();     // move servos, if necessary. 
        delay(6);
      }

      sequence = sequence + 10;
      break;
      ///////////////////////////// close grab

    case 30:

      delay(100);                    // recommended pause
      bioloid.loadPose(POS4);   // load the pose from FLASH, into the nextPose buffer
      bioloid.readPose();            // read in current servo positions to the curPose buffer
      Serial.println("###########################");
      Serial.println("Moving servos to Home position");
      Serial.println("###########################");    
      delay(1000);
      bioloid.interpolateSetup(1000); // setup for interpolation from current->next over 1/2 a second
      while(bioloid.interpolating > 0){  // do this while we have not reached our new pose
        bioloid.interpolateStep();     // move servos, if necessary. 
        delay(6);
      }

      sequence = sequence +10;
      break;
      //////////////////////////// move arm op

    case 40:

      delay(100);                    // recommended pause
      bioloid.loadPose(POS5);   // load the pose from FLASH, into the nextPose buffer
      bioloid.readPose();            // read in current servo positions to the curPose buffer
      Serial.println("###########################");
      Serial.println("Moving servos to Home position");
      Serial.println("###########################");    
      delay(1000);
      bioloid.interpolateSetup(1000); // setup for interpolation from current->next over 1/2 a second
      while(bioloid.interpolating > 0){  // do this while we have not reached our new pose
        bioloid.interpolateStep();     // move servos, if necessary. 
        delay(6);
      }

      sequence = sequence + 10;
      break;
      //////////////////////// move arm counter clock wurth

    case 50:

      delay(100);                    // recommended pause
      bioloid.loadPose(POS6);   // load the pose from FLASH, into the nextPose buffer
      bioloid.readPose();            // read in current servo positions to the curPose buffer
      Serial.println("###########################");
      Serial.println("Moving servos to Home position");
      Serial.println("###########################");    
      delay(1000);
      bioloid.interpolateSetup(1000); // setup for interpolation from current->next over 1/2 a second
      while(bioloid.interpolating > 0){  // do this while we have not reached our new pose
        bioloid.interpolateStep();     // move servos, if necessary. 
        delay(6);
      }
      sequence = sequence + 10;
     digitalWrite(pin4, HIGH); // sets pin 4 as high so the other robot knows we are ready.
     delay(1000);

     break;
      ////////////////////////////// 

   case 60:

      while (digitalRead(pin6) == LOW){ //If the other robot is ready = pin 6 is high, then the grapper will release
      delay(100);
      }
      digitalWrite(pin4, LOW);
        delay(100);                    // recommended pause
        bioloid.loadPose(POS7);   // load the pose from FLASH, into the nextPose buffer
        bioloid.readPose();            // read in current servo positions to the curPose buffer
        Serial.println("###########################");
        Serial.println("Moving servos to Home position");
        Serial.println("###########################");    
        delay(1000);
        bioloid.interpolateSetup(1000); // setup for interpolation from current->next over 1/2 a second
        while(bioloid.interpolating > 0){  // do this while we have not reached our new pose
          bioloid.interpolateStep();     // move servos, if necessary. 
          delay(6);
      }
      digitalWrite(pin4, HIGH);
      delay(3000);
      digitalWrite(pin4, LOW);     
      sequence = sequence + 10;
      break;
/*
    case 70:
      
    delay(100);                    // recommended pause
 bioloid.loadPose(POS8);   // load the pose from FLASH, into the nextPose buffer
 bioloid.readPose();            // read in current servo positions to the curPose buffer
 Serial.println("###########################");
 Serial.println("Moving servos to Home position");
 Serial.println("###########################");    
 delay(1000);
 bioloid.interpolateSetup(1000); // setup for interpolation from current->next over 1/2 a second
 while(bioloid.interpolating > 0){  // do this while we have not reached our new pose
 bioloid.interpolateStep();     // move servos, if necessary. 
 delay(6);
 }
sequence = sequence + 10;
      break;
    */
case 70:
//after the grapper is released, the arm returns to home position, sets sequence to 0 and sets pin4 low.

delay(100);  
  MoveHome();
      sequence = 0;
      }
    }
    }
